/**
 * 
 *   Copyright (c) 2010-2015, Csaba Debreceni, Istvan Rath and Daniel Varro
 *   This program and the accompanying materials are made available under the
 *   terms of the Eclipse Public License v. 2.0 which is available at
 *   http://www.eclipse.org/legal/epl-v20.html.
 *   
 *   SPDX-License-Identifier: EPL-2.0
 *  
 */
package org.eclipse.viatra.addon.viewers.runtime.model.patterns;

import java.util.Arrays;
import java.util.Collection;
import java.util.LinkedHashSet;
import java.util.List;
import java.util.Objects;
import java.util.Optional;
import java.util.Set;
import java.util.function.Consumer;
import java.util.stream.Collectors;
import java.util.stream.Stream;
import org.apache.log4j.Logger;
import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.EDataType;
import org.eclipse.viatra.addon.viewers.runtime.notation.Item;
import org.eclipse.viatra.query.runtime.api.IPatternMatch;
import org.eclipse.viatra.query.runtime.api.IQuerySpecification;
import org.eclipse.viatra.query.runtime.api.ViatraQueryEngine;
import org.eclipse.viatra.query.runtime.api.impl.BaseGeneratedEMFPQuery;
import org.eclipse.viatra.query.runtime.api.impl.BaseGeneratedEMFQuerySpecification;
import org.eclipse.viatra.query.runtime.api.impl.BaseMatcher;
import org.eclipse.viatra.query.runtime.api.impl.BasePatternMatch;
import org.eclipse.viatra.query.runtime.emf.types.EClassTransitiveInstancesKey;
import org.eclipse.viatra.query.runtime.emf.types.EDataTypeInSlotsKey;
import org.eclipse.viatra.query.runtime.emf.types.EStructuralFeatureInstancesKey;
import org.eclipse.viatra.query.runtime.matchers.backend.QueryEvaluationHint;
import org.eclipse.viatra.query.runtime.matchers.context.common.JavaTransitiveInstancesKey;
import org.eclipse.viatra.query.runtime.matchers.psystem.PBody;
import org.eclipse.viatra.query.runtime.matchers.psystem.PVariable;
import org.eclipse.viatra.query.runtime.matchers.psystem.basicdeferred.Equality;
import org.eclipse.viatra.query.runtime.matchers.psystem.basicdeferred.ExportedParameter;
import org.eclipse.viatra.query.runtime.matchers.psystem.basicdeferred.TypeFilterConstraint;
import org.eclipse.viatra.query.runtime.matchers.psystem.basicenumerables.TypeConstraint;
import org.eclipse.viatra.query.runtime.matchers.psystem.queries.PParameter;
import org.eclipse.viatra.query.runtime.matchers.psystem.queries.PParameterDirection;
import org.eclipse.viatra.query.runtime.matchers.psystem.queries.PVisibility;
import org.eclipse.viatra.query.runtime.matchers.tuple.Tuple;
import org.eclipse.viatra.query.runtime.matchers.tuple.Tuples;
import org.eclipse.viatra.query.runtime.util.ViatraQueryLoggingUtil;
import org.eclipse.viatra.transformation.views.traceability.Trace;

/**
 * A pattern-specific query specification that can instantiate Matcher in a type-safe way.
 * 
 * <p>Original source:
 *         <code><pre>
 *         pattern param2item(param : java Object, trace : Trace, item : Item) {
 *         	Trace.params(trace, param);
 *         	Trace.targets(trace, item);
 *         } or {
 *         	Trace.objects(trace, param);
 *         	Trace.targets(trace, item);
 *         }
 * </pre></code>
 * 
 * @see Matcher
 * @see Match
 * 
 */
@SuppressWarnings("all")
public final class Param2item extends BaseGeneratedEMFQuerySpecification<Param2item.Matcher> {
  /**
   * Pattern-specific match representation of the org.eclipse.viatra.addon.viewers.runtime.model.patterns.param2item pattern,
   * to be used in conjunction with {@link Matcher}.
   * 
   * <p>Class fields correspond to parameters of the pattern. Fields with value null are considered unassigned.
   * Each instance is a (possibly partial) substitution of pattern parameters,
   * usable to represent a match of the pattern in the result of a query,
   * or to specify the bound (fixed) input parameters when issuing a query.
   * 
   * @see Matcher
   * 
   */
  public static abstract class Match extends BasePatternMatch {
    private Object fParam;
    
    private Trace fTrace;
    
    private Item fItem;
    
    private static List<String> parameterNames = makeImmutableList("param", "trace", "item");
    
    private Match(final Object pParam, final Trace pTrace, final Item pItem) {
      this.fParam = pParam;
      this.fTrace = pTrace;
      this.fItem = pItem;
    }
    
    @Override
    public Object get(final String parameterName) {
      if ("param".equals(parameterName)) return this.fParam;
      if ("trace".equals(parameterName)) return this.fTrace;
      if ("item".equals(parameterName)) return this.fItem;
      return null;
    }
    
    public Object getParam() {
      return this.fParam;
    }
    
    public Trace getTrace() {
      return this.fTrace;
    }
    
    public Item getItem() {
      return this.fItem;
    }
    
    @Override
    public boolean set(final String parameterName, final Object newValue) {
      if (!isMutable()) throw new java.lang.UnsupportedOperationException();
      if ("param".equals(parameterName) && newValue instanceof Object) {
          this.fParam = (Object) newValue;
          return true;
      }
      if ("trace".equals(parameterName) ) {
          this.fTrace = (Trace) newValue;
          return true;
      }
      if ("item".equals(parameterName) ) {
          this.fItem = (Item) newValue;
          return true;
      }
      return false;
    }
    
    public void setParam(final Object pParam) {
      if (!isMutable()) throw new java.lang.UnsupportedOperationException();
      this.fParam = pParam;
    }
    
    public void setTrace(final Trace pTrace) {
      if (!isMutable()) throw new java.lang.UnsupportedOperationException();
      this.fTrace = pTrace;
    }
    
    public void setItem(final Item pItem) {
      if (!isMutable()) throw new java.lang.UnsupportedOperationException();
      this.fItem = pItem;
    }
    
    @Override
    public String patternName() {
      return "org.eclipse.viatra.addon.viewers.runtime.model.patterns.param2item";
    }
    
    @Override
    public List<String> parameterNames() {
      return Param2item.Match.parameterNames;
    }
    
    @Override
    public Object[] toArray() {
      return new Object[]{fParam, fTrace, fItem};
    }
    
    @Override
    public Param2item.Match toImmutable() {
      return isMutable() ? newMatch(fParam, fTrace, fItem) : this;
    }
    
    @Override
    public String prettyPrint() {
      StringBuilder result = new StringBuilder();
      result.append("\"param\"=" + prettyPrintValue(fParam) + ", ");
      result.append("\"trace\"=" + prettyPrintValue(fTrace) + ", ");
      result.append("\"item\"=" + prettyPrintValue(fItem));
      return result.toString();
    }
    
    @Override
    public int hashCode() {
      return Objects.hash(fParam, fTrace, fItem);
    }
    
    @Override
    public boolean equals(final Object obj) {
      if (this == obj)
          return true;
      if (obj == null) {
          return false;
      }
      if ((obj instanceof Param2item.Match)) {
          Param2item.Match other = (Param2item.Match) obj;
          return Objects.equals(fParam, other.fParam) && Objects.equals(fTrace, other.fTrace) && Objects.equals(fItem, other.fItem);
      } else {
          // this should be infrequent
          if (!(obj instanceof IPatternMatch)) {
              return false;
          }
          IPatternMatch otherSig  = (IPatternMatch) obj;
          return Objects.equals(specification(), otherSig.specification()) && Arrays.deepEquals(toArray(), otherSig.toArray());
      }
    }
    
    @Override
    public Param2item specification() {
      return Param2item.instance();
    }
    
    /**
     * Returns an empty, mutable match.
     * Fields of the mutable match can be filled to create a partial match, usable as matcher input.
     * 
     * @return the empty match.
     * 
     */
    public static Param2item.Match newEmptyMatch() {
      return new Mutable(null, null, null);
    }
    
    /**
     * Returns a mutable (partial) match.
     * Fields of the mutable match can be filled to create a partial match, usable as matcher input.
     * 
     * @param pParam the fixed value of pattern parameter param, or null if not bound.
     * @param pTrace the fixed value of pattern parameter trace, or null if not bound.
     * @param pItem the fixed value of pattern parameter item, or null if not bound.
     * @return the new, mutable (partial) match object.
     * 
     */
    public static Param2item.Match newMutableMatch(final Object pParam, final Trace pTrace, final Item pItem) {
      return new Mutable(pParam, pTrace, pItem);
    }
    
    /**
     * Returns a new (partial) match.
     * This can be used e.g. to call the matcher with a partial match.
     * <p>The returned match will be immutable. Use {@link #newEmptyMatch()} to obtain a mutable match object.
     * @param pParam the fixed value of pattern parameter param, or null if not bound.
     * @param pTrace the fixed value of pattern parameter trace, or null if not bound.
     * @param pItem the fixed value of pattern parameter item, or null if not bound.
     * @return the (partial) match object.
     * 
     */
    public static Param2item.Match newMatch(final Object pParam, final Trace pTrace, final Item pItem) {
      return new Immutable(pParam, pTrace, pItem);
    }
    
    private static final class Mutable extends Param2item.Match {
      Mutable(final Object pParam, final Trace pTrace, final Item pItem) {
        super(pParam, pTrace, pItem);
      }
      
      @Override
      public boolean isMutable() {
        return true;
      }
    }
    
    private static final class Immutable extends Param2item.Match {
      Immutable(final Object pParam, final Trace pTrace, final Item pItem) {
        super(pParam, pTrace, pItem);
      }
      
      @Override
      public boolean isMutable() {
        return false;
      }
    }
  }
  
  /**
   * Generated pattern matcher API of the org.eclipse.viatra.addon.viewers.runtime.model.patterns.param2item pattern,
   * providing pattern-specific query methods.
   * 
   * <p>Use the pattern matcher on a given model via {@link #on(ViatraQueryEngine)},
   * e.g. in conjunction with {@link ViatraQueryEngine#on(QueryScope)}.
   * 
   * <p>Matches of the pattern will be represented as {@link Match}.
   * 
   * <p>Original source:
   * <code><pre>
   * pattern param2item(param : java Object, trace : Trace, item : Item) {
   * 	Trace.params(trace, param);
   * 	Trace.targets(trace, item);
   * } or {
   * 	Trace.objects(trace, param);
   * 	Trace.targets(trace, item);
   * }
   * </pre></code>
   * 
   * @see Match
   * @see Param2item
   * 
   */
  public static class Matcher extends BaseMatcher<Param2item.Match> {
    /**
     * Initializes the pattern matcher within an existing VIATRA Query engine.
     * If the pattern matcher is already constructed in the engine, only a light-weight reference is returned.
     * 
     * @param engine the existing VIATRA Query engine in which this matcher will be created.
     * @throws ViatraQueryRuntimeException if an error occurs during pattern matcher creation
     * 
     */
    public static Param2item.Matcher on(final ViatraQueryEngine engine) {
      // check if matcher already exists
      Matcher matcher = engine.getExistingMatcher(querySpecification());
      if (matcher == null) {
          matcher = (Matcher)engine.getMatcher(querySpecification());
      }
      return matcher;
    }
    
    /**
     * @throws ViatraQueryRuntimeException if an error occurs during pattern matcher creation
     * @return an initialized matcher
     * @noreference This method is for internal matcher initialization by the framework, do not call it manually.
     * 
     */
    public static Param2item.Matcher create() {
      return new Matcher();
    }
    
    private static final int POSITION_PARAM = 0;
    
    private static final int POSITION_TRACE = 1;
    
    private static final int POSITION_ITEM = 2;
    
    private static final Logger LOGGER = ViatraQueryLoggingUtil.getLogger(Param2item.Matcher.class);
    
    /**
     * Initializes the pattern matcher within an existing VIATRA Query engine.
     * If the pattern matcher is already constructed in the engine, only a light-weight reference is returned.
     * 
     * @param engine the existing VIATRA Query engine in which this matcher will be created.
     * @throws ViatraQueryRuntimeException if an error occurs during pattern matcher creation
     * 
     */
    private Matcher() {
      super(querySpecification());
    }
    
    /**
     * Returns the set of all matches of the pattern that conform to the given fixed values of some parameters.
     * @param pParam the fixed value of pattern parameter param, or null if not bound.
     * @param pTrace the fixed value of pattern parameter trace, or null if not bound.
     * @param pItem the fixed value of pattern parameter item, or null if not bound.
     * @return matches represented as a Match object.
     * 
     */
    public Collection<Param2item.Match> getAllMatches(final Object pParam, final Trace pTrace, final Item pItem) {
      return rawStreamAllMatches(new Object[]{pParam, pTrace, pItem}).collect(Collectors.toSet());
    }
    
    /**
     * Returns a stream of all matches of the pattern that conform to the given fixed values of some parameters.
     * </p>
     * <strong>NOTE</strong>: It is important not to modify the source model while the stream is being processed.
     * If the match set of the pattern changes during processing, the contents of the stream is <strong>undefined</strong>.
     * In such cases, either rely on {@link #getAllMatches()} or collect the results of the stream in end-user code.
     * @param pParam the fixed value of pattern parameter param, or null if not bound.
     * @param pTrace the fixed value of pattern parameter trace, or null if not bound.
     * @param pItem the fixed value of pattern parameter item, or null if not bound.
     * @return a stream of matches represented as a Match object.
     * 
     */
    public Stream<Param2item.Match> streamAllMatches(final Object pParam, final Trace pTrace, final Item pItem) {
      return rawStreamAllMatches(new Object[]{pParam, pTrace, pItem});
    }
    
    /**
     * Returns an arbitrarily chosen match of the pattern that conforms to the given fixed values of some parameters.
     * Neither determinism nor randomness of selection is guaranteed.
     * @param pParam the fixed value of pattern parameter param, or null if not bound.
     * @param pTrace the fixed value of pattern parameter trace, or null if not bound.
     * @param pItem the fixed value of pattern parameter item, or null if not bound.
     * @return a match represented as a Match object, or null if no match is found.
     * 
     */
    public Optional<Param2item.Match> getOneArbitraryMatch(final Object pParam, final Trace pTrace, final Item pItem) {
      return rawGetOneArbitraryMatch(new Object[]{pParam, pTrace, pItem});
    }
    
    /**
     * Indicates whether the given combination of specified pattern parameters constitute a valid pattern match,
     * under any possible substitution of the unspecified parameters (if any).
     * @param pParam the fixed value of pattern parameter param, or null if not bound.
     * @param pTrace the fixed value of pattern parameter trace, or null if not bound.
     * @param pItem the fixed value of pattern parameter item, or null if not bound.
     * @return true if the input is a valid (partial) match of the pattern.
     * 
     */
    public boolean hasMatch(final Object pParam, final Trace pTrace, final Item pItem) {
      return rawHasMatch(new Object[]{pParam, pTrace, pItem});
    }
    
    /**
     * Returns the number of all matches of the pattern that conform to the given fixed values of some parameters.
     * @param pParam the fixed value of pattern parameter param, or null if not bound.
     * @param pTrace the fixed value of pattern parameter trace, or null if not bound.
     * @param pItem the fixed value of pattern parameter item, or null if not bound.
     * @return the number of pattern matches found.
     * 
     */
    public int countMatches(final Object pParam, final Trace pTrace, final Item pItem) {
      return rawCountMatches(new Object[]{pParam, pTrace, pItem});
    }
    
    /**
     * Executes the given processor on an arbitrarily chosen match of the pattern that conforms to the given fixed values of some parameters.
     * Neither determinism nor randomness of selection is guaranteed.
     * @param pParam the fixed value of pattern parameter param, or null if not bound.
     * @param pTrace the fixed value of pattern parameter trace, or null if not bound.
     * @param pItem the fixed value of pattern parameter item, or null if not bound.
     * @param processor the action that will process the selected match.
     * @return true if the pattern has at least one match with the given parameter values, false if the processor was not invoked
     * 
     */
    public boolean forOneArbitraryMatch(final Object pParam, final Trace pTrace, final Item pItem, final Consumer<? super Param2item.Match> processor) {
      return rawForOneArbitraryMatch(new Object[]{pParam, pTrace, pItem}, processor);
    }
    
    /**
     * Returns a new (partial) match.
     * This can be used e.g. to call the matcher with a partial match.
     * <p>The returned match will be immutable. Use {@link #newEmptyMatch()} to obtain a mutable match object.
     * @param pParam the fixed value of pattern parameter param, or null if not bound.
     * @param pTrace the fixed value of pattern parameter trace, or null if not bound.
     * @param pItem the fixed value of pattern parameter item, or null if not bound.
     * @return the (partial) match object.
     * 
     */
    public Param2item.Match newMatch(final Object pParam, final Trace pTrace, final Item pItem) {
      return Param2item.Match.newMatch(pParam, pTrace, pItem);
    }
    
    /**
     * Retrieve the set of values that occur in matches for param.
     * @return the Set of all values or empty set if there are no matches
     * 
     */
    protected Stream<Object> rawStreamAllValuesOfparam(final Object[] parameters) {
      return rawStreamAllValues(POSITION_PARAM, parameters).map(Object.class::cast);
    }
    
    /**
     * Retrieve the set of values that occur in matches for param.
     * @return the Set of all values or empty set if there are no matches
     * 
     */
    public Set<Object> getAllValuesOfparam() {
      return rawStreamAllValuesOfparam(emptyArray()).collect(Collectors.toSet());
    }
    
    /**
     * Retrieve the set of values that occur in matches for param.
     * @return the Set of all values or empty set if there are no matches
     * 
     */
    public Stream<Object> streamAllValuesOfparam() {
      return rawStreamAllValuesOfparam(emptyArray());
    }
    
    /**
     * Retrieve the set of values that occur in matches for param.
     * </p>
     * <strong>NOTE</strong>: It is important not to modify the source model while the stream is being processed.
     * If the match set of the pattern changes during processing, the contents of the stream is <strong>undefined</strong>.
     * In such cases, either rely on {@link #getAllMatches()} or collect the results of the stream in end-user code.
     *      
     * @return the Stream of all values or empty set if there are no matches
     * 
     */
    public Stream<Object> streamAllValuesOfparam(final Param2item.Match partialMatch) {
      return rawStreamAllValuesOfparam(partialMatch.toArray());
    }
    
    /**
     * Retrieve the set of values that occur in matches for param.
     * </p>
     * <strong>NOTE</strong>: It is important not to modify the source model while the stream is being processed.
     * If the match set of the pattern changes during processing, the contents of the stream is <strong>undefined</strong>.
     * In such cases, either rely on {@link #getAllMatches()} or collect the results of the stream in end-user code.
     *      
     * @return the Stream of all values or empty set if there are no matches
     * 
     */
    public Stream<Object> streamAllValuesOfparam(final Trace pTrace, final Item pItem) {
      return rawStreamAllValuesOfparam(new Object[]{null, pTrace, pItem});
    }
    
    /**
     * Retrieve the set of values that occur in matches for param.
     * @return the Set of all values or empty set if there are no matches
     * 
     */
    public Set<Object> getAllValuesOfparam(final Param2item.Match partialMatch) {
      return rawStreamAllValuesOfparam(partialMatch.toArray()).collect(Collectors.toSet());
    }
    
    /**
     * Retrieve the set of values that occur in matches for param.
     * @return the Set of all values or empty set if there are no matches
     * 
     */
    public Set<Object> getAllValuesOfparam(final Trace pTrace, final Item pItem) {
      return rawStreamAllValuesOfparam(new Object[]{null, pTrace, pItem}).collect(Collectors.toSet());
    }
    
    /**
     * Retrieve the set of values that occur in matches for trace.
     * @return the Set of all values or empty set if there are no matches
     * 
     */
    protected Stream<Trace> rawStreamAllValuesOftrace(final Object[] parameters) {
      return rawStreamAllValues(POSITION_TRACE, parameters).map(Trace.class::cast);
    }
    
    /**
     * Retrieve the set of values that occur in matches for trace.
     * @return the Set of all values or empty set if there are no matches
     * 
     */
    public Set<Trace> getAllValuesOftrace() {
      return rawStreamAllValuesOftrace(emptyArray()).collect(Collectors.toSet());
    }
    
    /**
     * Retrieve the set of values that occur in matches for trace.
     * @return the Set of all values or empty set if there are no matches
     * 
     */
    public Stream<Trace> streamAllValuesOftrace() {
      return rawStreamAllValuesOftrace(emptyArray());
    }
    
    /**
     * Retrieve the set of values that occur in matches for trace.
     * </p>
     * <strong>NOTE</strong>: It is important not to modify the source model while the stream is being processed.
     * If the match set of the pattern changes during processing, the contents of the stream is <strong>undefined</strong>.
     * In such cases, either rely on {@link #getAllMatches()} or collect the results of the stream in end-user code.
     *      
     * @return the Stream of all values or empty set if there are no matches
     * 
     */
    public Stream<Trace> streamAllValuesOftrace(final Param2item.Match partialMatch) {
      return rawStreamAllValuesOftrace(partialMatch.toArray());
    }
    
    /**
     * Retrieve the set of values that occur in matches for trace.
     * </p>
     * <strong>NOTE</strong>: It is important not to modify the source model while the stream is being processed.
     * If the match set of the pattern changes during processing, the contents of the stream is <strong>undefined</strong>.
     * In such cases, either rely on {@link #getAllMatches()} or collect the results of the stream in end-user code.
     *      
     * @return the Stream of all values or empty set if there are no matches
     * 
     */
    public Stream<Trace> streamAllValuesOftrace(final Object pParam, final Item pItem) {
      return rawStreamAllValuesOftrace(new Object[]{pParam, null, pItem});
    }
    
    /**
     * Retrieve the set of values that occur in matches for trace.
     * @return the Set of all values or empty set if there are no matches
     * 
     */
    public Set<Trace> getAllValuesOftrace(final Param2item.Match partialMatch) {
      return rawStreamAllValuesOftrace(partialMatch.toArray()).collect(Collectors.toSet());
    }
    
    /**
     * Retrieve the set of values that occur in matches for trace.
     * @return the Set of all values or empty set if there are no matches
     * 
     */
    public Set<Trace> getAllValuesOftrace(final Object pParam, final Item pItem) {
      return rawStreamAllValuesOftrace(new Object[]{pParam, null, pItem}).collect(Collectors.toSet());
    }
    
    /**
     * Retrieve the set of values that occur in matches for item.
     * @return the Set of all values or empty set if there are no matches
     * 
     */
    protected Stream<Item> rawStreamAllValuesOfitem(final Object[] parameters) {
      return rawStreamAllValues(POSITION_ITEM, parameters).map(Item.class::cast);
    }
    
    /**
     * Retrieve the set of values that occur in matches for item.
     * @return the Set of all values or empty set if there are no matches
     * 
     */
    public Set<Item> getAllValuesOfitem() {
      return rawStreamAllValuesOfitem(emptyArray()).collect(Collectors.toSet());
    }
    
    /**
     * Retrieve the set of values that occur in matches for item.
     * @return the Set of all values or empty set if there are no matches
     * 
     */
    public Stream<Item> streamAllValuesOfitem() {
      return rawStreamAllValuesOfitem(emptyArray());
    }
    
    /**
     * Retrieve the set of values that occur in matches for item.
     * </p>
     * <strong>NOTE</strong>: It is important not to modify the source model while the stream is being processed.
     * If the match set of the pattern changes during processing, the contents of the stream is <strong>undefined</strong>.
     * In such cases, either rely on {@link #getAllMatches()} or collect the results of the stream in end-user code.
     *      
     * @return the Stream of all values or empty set if there are no matches
     * 
     */
    public Stream<Item> streamAllValuesOfitem(final Param2item.Match partialMatch) {
      return rawStreamAllValuesOfitem(partialMatch.toArray());
    }
    
    /**
     * Retrieve the set of values that occur in matches for item.
     * </p>
     * <strong>NOTE</strong>: It is important not to modify the source model while the stream is being processed.
     * If the match set of the pattern changes during processing, the contents of the stream is <strong>undefined</strong>.
     * In such cases, either rely on {@link #getAllMatches()} or collect the results of the stream in end-user code.
     *      
     * @return the Stream of all values or empty set if there are no matches
     * 
     */
    public Stream<Item> streamAllValuesOfitem(final Object pParam, final Trace pTrace) {
      return rawStreamAllValuesOfitem(new Object[]{pParam, pTrace, null});
    }
    
    /**
     * Retrieve the set of values that occur in matches for item.
     * @return the Set of all values or empty set if there are no matches
     * 
     */
    public Set<Item> getAllValuesOfitem(final Param2item.Match partialMatch) {
      return rawStreamAllValuesOfitem(partialMatch.toArray()).collect(Collectors.toSet());
    }
    
    /**
     * Retrieve the set of values that occur in matches for item.
     * @return the Set of all values or empty set if there are no matches
     * 
     */
    public Set<Item> getAllValuesOfitem(final Object pParam, final Trace pTrace) {
      return rawStreamAllValuesOfitem(new Object[]{pParam, pTrace, null}).collect(Collectors.toSet());
    }
    
    @Override
    protected Param2item.Match tupleToMatch(final Tuple t) {
      try {
          return Param2item.Match.newMatch((Object) t.get(POSITION_PARAM), (Trace) t.get(POSITION_TRACE), (Item) t.get(POSITION_ITEM));
      } catch(ClassCastException e) {
          LOGGER.error("Element(s) in tuple not properly typed!",e);
          return null;
      }
    }
    
    @Override
    protected Param2item.Match arrayToMatch(final Object[] match) {
      try {
          return Param2item.Match.newMatch((Object) match[POSITION_PARAM], (Trace) match[POSITION_TRACE], (Item) match[POSITION_ITEM]);
      } catch(ClassCastException e) {
          LOGGER.error("Element(s) in array not properly typed!",e);
          return null;
      }
    }
    
    @Override
    protected Param2item.Match arrayToMatchMutable(final Object[] match) {
      try {
          return Param2item.Match.newMutableMatch((Object) match[POSITION_PARAM], (Trace) match[POSITION_TRACE], (Item) match[POSITION_ITEM]);
      } catch(ClassCastException e) {
          LOGGER.error("Element(s) in array not properly typed!",e);
          return null;
      }
    }
    
    /**
     * @return the singleton instance of the query specification of this pattern
     * @throws ViatraQueryRuntimeException if the pattern definition could not be loaded
     * 
     */
    public static IQuerySpecification<Param2item.Matcher> querySpecification() {
      return Param2item.instance();
    }
  }
  
  private Param2item() {
    super(GeneratedPQuery.INSTANCE);
  }
  
  /**
   * @return the singleton instance of the query specification
   * @throws ViatraQueryRuntimeException if the pattern definition could not be loaded
   * 
   */
  public static Param2item instance() {
    try{
        return LazyHolder.INSTANCE;
    } catch (ExceptionInInitializerError err) {
        throw processInitializerError(err);
    }
  }
  
  @Override
  protected Param2item.Matcher instantiate(final ViatraQueryEngine engine) {
    return Param2item.Matcher.on(engine);
  }
  
  @Override
  public Param2item.Matcher instantiate() {
    return Param2item.Matcher.create();
  }
  
  @Override
  public Param2item.Match newEmptyMatch() {
    return Param2item.Match.newEmptyMatch();
  }
  
  @Override
  public Param2item.Match newMatch(final Object... parameters) {
    return Param2item.Match.newMatch((java.lang.Object) parameters[0], (org.eclipse.viatra.transformation.views.traceability.Trace) parameters[1], (org.eclipse.viatra.addon.viewers.runtime.notation.Item) parameters[2]);
  }
  
  /**
   * Inner class allowing the singleton instance of {@link JvmGenericType: org.eclipse.viatra.addon.viewers.runtime.model.patterns.Param2item (visibility: PUBLIC, simpleName: Param2item, identifier: org.eclipse.viatra.addon.viewers.runtime.model.patterns.Param2item, deprecated: <unset>) (abstract: false, static: false, final: true, packageName: org.eclipse.viatra.addon.viewers.runtime.model.patterns) (interface: false, strictFloatingPoint: false, anonymous: false)} to be created 
   *     <b>not</b> at the class load time of the outer class, 
   *     but rather at the first call to {@link JvmGenericType: org.eclipse.viatra.addon.viewers.runtime.model.patterns.Param2item (visibility: PUBLIC, simpleName: Param2item, identifier: org.eclipse.viatra.addon.viewers.runtime.model.patterns.Param2item, deprecated: <unset>) (abstract: false, static: false, final: true, packageName: org.eclipse.viatra.addon.viewers.runtime.model.patterns) (interface: false, strictFloatingPoint: false, anonymous: false)#instance()}.
   * 
   * <p> This workaround is required e.g. to support recursion.
   * 
   */
  private static class LazyHolder {
    private static final Param2item INSTANCE = new Param2item();
    
    /**
     * Statically initializes the query specification <b>after</b> the field {@link #INSTANCE} is assigned.
     * This initialization order is required to support indirect recursion.
     * 
     * <p> The static initializer is defined using a helper field to work around limitations of the code generator.
     * 
     */
    private static final Object STATIC_INITIALIZER = ensureInitialized();
    
    public static Object ensureInitialized() {
      INSTANCE.ensureInitializedInternal();
      return null;
    }
  }
  
  private static class GeneratedPQuery extends BaseGeneratedEMFPQuery {
    private static final Param2item.GeneratedPQuery INSTANCE = new GeneratedPQuery();
    
    private final PParameter parameter_param = new PParameter("param", "java.lang.Object", new JavaTransitiveInstancesKey(java.lang.Object.class), PParameterDirection.INOUT);
    
    private final PParameter parameter_trace = new PParameter("trace", "org.eclipse.viatra.transformation.views.traceability.Trace", new EClassTransitiveInstancesKey((EClass)getClassifierLiteralSafe("http://www.eclipse.org/viatra/transformation/views/traceability/1.0", "Trace")), PParameterDirection.INOUT);
    
    private final PParameter parameter_item = new PParameter("item", "org.eclipse.viatra.addon.viewers.runtime.notation.Item", new EClassTransitiveInstancesKey((EClass)getClassifierLiteralSafe("http://www.eclipse.org/viatra/addon/viewers/notation/1.0", "Item")), PParameterDirection.INOUT);
    
    private final List<PParameter> parameters = Arrays.asList(parameter_param, parameter_trace, parameter_item);
    
    private GeneratedPQuery() {
      super(PVisibility.PUBLIC);
    }
    
    @Override
    public String getFullyQualifiedName() {
      return "org.eclipse.viatra.addon.viewers.runtime.model.patterns.param2item";
    }
    
    @Override
    public List<String> getParameterNames() {
      return Arrays.asList("param","trace","item");
    }
    
    @Override
    public List<PParameter> getParameters() {
      return parameters;
    }
    
    @Override
    public Set<PBody> doGetContainedBodies() {
      setEvaluationHints(new QueryEvaluationHint(null, QueryEvaluationHint.BackendRequirement.UNSPECIFIED));
      Set<PBody> bodies = new LinkedHashSet<>();
      {
          PBody body = new PBody(this);
          PVariable var_param = body.getOrCreateVariableByName("param");
          PVariable var_trace = body.getOrCreateVariableByName("trace");
          PVariable var_item = body.getOrCreateVariableByName("item");
          new TypeFilterConstraint(body, Tuples.flatTupleOf(var_param), new JavaTransitiveInstancesKey(java.lang.Object.class));
          new TypeConstraint(body, Tuples.flatTupleOf(var_trace), new EClassTransitiveInstancesKey((EClass)getClassifierLiteral("http://www.eclipse.org/viatra/transformation/views/traceability/1.0", "Trace")));
          new TypeConstraint(body, Tuples.flatTupleOf(var_item), new EClassTransitiveInstancesKey((EClass)getClassifierLiteral("http://www.eclipse.org/viatra/addon/viewers/notation/1.0", "Item")));
          body.setSymbolicParameters(Arrays.<ExportedParameter>asList(
             new ExportedParameter(body, var_param, parameter_param),
             new ExportedParameter(body, var_trace, parameter_trace),
             new ExportedParameter(body, var_item, parameter_item)
          ));
          // 	Trace.params(trace, param)
          new TypeConstraint(body, Tuples.flatTupleOf(var_trace), new EClassTransitiveInstancesKey((EClass)getClassifierLiteral("http://www.eclipse.org/viatra/transformation/views/traceability/1.0", "Trace")));
          PVariable var__virtual_0_ = body.getOrCreateVariableByName(".virtual{0}");
          new TypeConstraint(body, Tuples.flatTupleOf(var_trace, var__virtual_0_), new EStructuralFeatureInstancesKey(getFeatureLiteral("http://www.eclipse.org/viatra/transformation/views/traceability/1.0", "Trace", "params")));
          new TypeConstraint(body, Tuples.flatTupleOf(var__virtual_0_), new EClassTransitiveInstancesKey((EClass)getClassifierLiteral("http://www.eclipse.org/emf/2002/Ecore", "EObject")));
          new Equality(body, var__virtual_0_, var_param);
          // 	Trace.targets(trace, item)
          new TypeConstraint(body, Tuples.flatTupleOf(var_trace), new EClassTransitiveInstancesKey((EClass)getClassifierLiteral("http://www.eclipse.org/viatra/transformation/views/traceability/1.0", "Trace")));
          PVariable var__virtual_1_ = body.getOrCreateVariableByName(".virtual{1}");
          new TypeConstraint(body, Tuples.flatTupleOf(var_trace, var__virtual_1_), new EStructuralFeatureInstancesKey(getFeatureLiteral("http://www.eclipse.org/viatra/transformation/views/traceability/1.0", "Trace", "targets")));
          new TypeConstraint(body, Tuples.flatTupleOf(var__virtual_1_), new EClassTransitiveInstancesKey((EClass)getClassifierLiteral("http://www.eclipse.org/emf/2002/Ecore", "EObject")));
          new Equality(body, var__virtual_1_, var_item);
          bodies.add(body);
      }
      {
          PBody body = new PBody(this);
          PVariable var_param = body.getOrCreateVariableByName("param");
          PVariable var_trace = body.getOrCreateVariableByName("trace");
          PVariable var_item = body.getOrCreateVariableByName("item");
          new TypeFilterConstraint(body, Tuples.flatTupleOf(var_param), new JavaTransitiveInstancesKey(java.lang.Object.class));
          new TypeConstraint(body, Tuples.flatTupleOf(var_trace), new EClassTransitiveInstancesKey((EClass)getClassifierLiteral("http://www.eclipse.org/viatra/transformation/views/traceability/1.0", "Trace")));
          new TypeConstraint(body, Tuples.flatTupleOf(var_item), new EClassTransitiveInstancesKey((EClass)getClassifierLiteral("http://www.eclipse.org/viatra/addon/viewers/notation/1.0", "Item")));
          body.setSymbolicParameters(Arrays.<ExportedParameter>asList(
             new ExportedParameter(body, var_param, parameter_param),
             new ExportedParameter(body, var_trace, parameter_trace),
             new ExportedParameter(body, var_item, parameter_item)
          ));
          // 	Trace.objects(trace, param)
          new TypeConstraint(body, Tuples.flatTupleOf(var_trace), new EClassTransitiveInstancesKey((EClass)getClassifierLiteral("http://www.eclipse.org/viatra/transformation/views/traceability/1.0", "Trace")));
          PVariable var__virtual_0_ = body.getOrCreateVariableByName(".virtual{0}");
          new TypeConstraint(body, Tuples.flatTupleOf(var_trace, var__virtual_0_), new EStructuralFeatureInstancesKey(getFeatureLiteral("http://www.eclipse.org/viatra/transformation/views/traceability/1.0", "Trace", "objects")));
          new TypeConstraint(body, Tuples.flatTupleOf(var__virtual_0_), new EDataTypeInSlotsKey((EDataType)getClassifierLiteral("http://www.eclipse.org/emf/2002/Ecore", "EJavaObject")));
          new Equality(body, var__virtual_0_, var_param);
          // 	Trace.targets(trace, item)
          new TypeConstraint(body, Tuples.flatTupleOf(var_trace), new EClassTransitiveInstancesKey((EClass)getClassifierLiteral("http://www.eclipse.org/viatra/transformation/views/traceability/1.0", "Trace")));
          PVariable var__virtual_1_ = body.getOrCreateVariableByName(".virtual{1}");
          new TypeConstraint(body, Tuples.flatTupleOf(var_trace, var__virtual_1_), new EStructuralFeatureInstancesKey(getFeatureLiteral("http://www.eclipse.org/viatra/transformation/views/traceability/1.0", "Trace", "targets")));
          new TypeConstraint(body, Tuples.flatTupleOf(var__virtual_1_), new EClassTransitiveInstancesKey((EClass)getClassifierLiteral("http://www.eclipse.org/emf/2002/Ecore", "EObject")));
          new Equality(body, var__virtual_1_, var_item);
          bodies.add(body);
      }
      return bodies;
    }
  }
}
